package org.malnatij.svplugin.views.actions;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.Separator;
import org.malnatij.svplugin.XRay;
import org.malnatij.svplugin.model.ProjectRepresentation;
import org.malnatij.svplugin.model.core.ModelNotPreviouslyScheduled;
import org.malnatij.svplugin.util.Log;
import org.malnatij.svplugin.views.XRayWorkbenchView;

/**
 * A set of actions that will be added to the view
 * @author Jacopo Malnati
 *
 */
public class ToolbarActions extends XRay {
	private XRayWorkbenchView viewContainer;
	// a string representing the project
	private Action infoTrickAction = null;
	
	private IToolBarManager mgr = null;
	
	public ToolbarActions(XRayWorkbenchView viewContainer){
		this.viewContainer = viewContainer;
		// get the manager
		mgr = viewContainer.getViewSite().getActionBars().getToolBarManager();
	}
	
	public void createActions(){
		Log.printSVPluginView("Creating actions");
		
		createInfoTrickAction();
		
		// other inner actions..
	}
	
	/**
	 * Create tool-bar.
	 */
	public void createToolbar() {
		mgr.add(new RefreshAction(viewContainer, this));
		mgr.add(new SnapShotAction(viewContainer));
		mgr.add(new OpenSelectedClassesAction(viewContainer));
		mgr.add(new Separator());
		mgr.add(new PackageFilterAction(viewContainer));
		mgr.add(new HideNodeAction(viewContainer));
		mgr.add(new SelectionAction(viewContainer));
		mgr.add(new ColorTagAction(viewContainer));
		mgr.add(new ZoomAction(viewContainer));
		mgr.add(new EdgesCreatorAction(viewContainer));
		mgr.add(new SelectViewAction(viewContainer));
		mgr.update(true);
	}
	
	public void createInfoTrickAction() {
		ProjectRepresentation theProject;
		try {
			theProject = viewContainer.getModeledProject();
			infoTrickAction = new Action(theProject.toShortString()) {
				public void run() {
					// nothing it's just a trick to show text in the action bar
				}
			};
			mgr.add(infoTrickAction);
			mgr.update(true);
		} catch (ModelNotPreviouslyScheduled e) {
			// ignore the request
			return;
		}
	}

	protected Action getInfoTrickAction() {
		return infoTrickAction;
	}

}
